"use strict";
const {app, protocol, dialog, ipcMain, Menu} = require("electron");
const path                                   = require("path");
const fs                                     = require("fs/promises");
const fsn                                    = require("fs");

const AppInfo         = require("../package.json");
const WindowManager   = require("./WindowManager");
const FontManager     = require("./FontManager");
const SettingsManager = require("./SettingsManager");
const MainMenu        = require("./MainMenu");
const {parser}        = require("./Parser");

app.commandLine.appendSwitch("disable-http-cache");

const RECENT_FILES = 5;
class Ation {
	windowManager;
	fontManager;
	settingsManager;
	mainMenu;
	watcher;
	currentFile;
	fileToOpen;
	recentFiles;

	constructor() {
		if (Ation.Instances > 0)
			throw new Error("Only one Instance of Ation possible");
		this.fileToOpen      = null;
		this.currentFile     = "";
		this.watcher         = null;
		this.windowManager   = new WindowManager(this);
		this.fontManager     = new FontManager();
		this.settingsManager = new SettingsManager(this);
		this.recentFiles     = this.settingsManager.get("recentFiles", []);
		this.mainMenu        = new MainMenu(this);

		ipcMain.handle("Ation::appVersion", () => AppInfo.version);
		ipcMain.handle("Ation::saveFile", async (_, newContent) => this.saveFile(newContent));
		ipcMain.on("Ation::closeFile",  () => this.closeFile());

		app.on("open-file", (_, path) => {
			this.fileToOpen = path;
		});

		app.whenReady().then(async () => {
			this.settingsManager.change();

			// this is a hack for windows, because they do not send the
			// `open-file` event, but pass a "cli parameter" as second
			// argument
			if (!this.fileToOpen) {
				if (process.argv.length >= 2)
					this.fileToOpen = process.argv[1];
			}

			if (this.fileToOpen)
				this.openFile(this.fileToOpen);
			protocol.registerFileProtocol("slideimg", (request, callback) => {
				const uri       = request.url.replace(/^slideimg:\/\//, "");
				const extension = path.extname(uri).replace(/^\./, "");

				if (["png", "jpg", "jpeg", "svg", "bmp", "gif", "tiff"].includes(extension.toLowerCase()))
					callback(uri);
			});

		});
	}

	async saveFile(newContent) {
		return fs.writeFile(this.currentFile, newContent, { encoding : "utf-8" })
	}

	addRecentFile(filePath) {
		if (this.recentFiles.includes(filePath)) {
			const position = this.recentFiles.indexOf(filePath);

			this.recentFiles.splice(position, 1),
			this.recentFiles.push(filePath);
		} else if (this.recentFiles.length < RECENT_FILES) {
			this.recentFiles.push(filePath);
		} else {
			this.recentFiles.shift();
			this.recentFiles.push(filePath);
		}

		this.settingsManager.set("recentFiles", this.recentFiles);
		this.mainMenu.refresh();
	}

	async openFile(filePath = null, change = false) {
			if (!this.windowManager.mainWindow)
				return null;

			if (!filePath) {
				const result = await dialog.showOpenDialog(this.windowManager.mainWindow, {
					title   : "open file",
					filters : [
						{
							name       : "Markdown files",
							extensions : ["md"]
						}
					]
				});

				if (result.canceled || result.filePaths.length < 1)
					return;
				filePath = result.filePaths[0];
			}

			try {
				const fileContents = await fs.readFile(filePath, {encoding : "utf-8"});
				const basePath     = path.dirname(filePath);
				const data         = parser(fileContents);

				if (!change) 
					this.currentFile = filePath;
				if (this.watcher)
					this.watcher.close();
				this.watcher = fsn.watch(filePath, (eventType, path) => {
					if (path.match(/~$/)) // handling unix swap files here
						setTimeout(() => this.openFile(this.currentFile, true), 1000);
					else
						this.openFile(this.currentFile, true);
				});
				this.windowManager.mainWindow.send("Ation::openFile", [basePath, data, fileContents]);
				this.addRecentFile(filePath);
				Menu.getApplicationMenu().getMenuItemById("close-file").enabled = this.currentFile !== "";
			} catch (error) {
				console.log(error);
				return;
			}
	}

	closeFile() {
		this.currentFile = "";
		if (this.watcher)
			this.watcher.close();
		this.watcher = null;
		this.windowManager.mainWindow.send("Ation::closeFile");
		Menu.getApplicationMenu().getMenuItemById("close-file").enabled = this.currentFile !== "";
	}
}
Ation.Instances = 0;

module.exports = Ation;