import React, {useMemo, useContext} from "react";

import SyntaxHighlighter from "react-syntax-highlighter"
import {monokai}         from "react-syntax-highlighter/dist/esm/styles/hljs";

import SlideContext from "../shared/SlideContext";

const EscapeSequences = {
	"&quot;" : '"',
	"&lt;"   : "<",
	"&gt;"   : ">",
	"&amp;"  : "&",
	"&#39;"  : "'"
};
const EscapeSequencesRegex = new RegExp(Object.keys(EscapeSequences).join('|'), "g");

const Children = ({items}) => {
	if (items instanceof Array)
		return <>{items.map((child, index) => <SlideItem item={child} key={index} />)}</>;
	return null;
};

const SlideItem = ({item}) => {
	const {basePath} = useContext(SlideContext);

	const processEscapeSequences = text => {
		return text.replace(EscapeSequencesRegex, match => EscapeSequences[match]);
	};

	const content = useMemo(() => {
		switch (item.type) {
			case "heading":
				const level = item.level || item.depth;
				switch(level) {
					case 1:
						return <h1><Children items={item.tokens} /></h1>
					case 2:
						return <h2><Children items={item.tokens} /></h2>
					case 3:
						return <h3><Children items={item.tokens} /></h3>
					case 4:
						return <h4><Children items={item.tokens} /></h4>
					case 5:
					default:
						console.log("UNKNOWN LEVEL", item);
						return <h5><Children items={item.tokens} /></h5>
				}
			case "list":
				if (item.ordered)
					return <ol><Children items={item.items} /></ol>
				else
					return <ul><Children items={item.items} /></ul>
			case "list_item":
				const prefix = item.task ? <span className={`checkbox${item.checked ? " checked" : ""}`}></span> : null;
				return <li className={item.task ? "task" : ""}>{prefix} <Children items={item.tokens} /></li>
			case "image":
				return (
					<span className="figure">
						<img src={`slideimg://${basePath}/${item.href.replace(/\.\//, "")}`} alt={item.text} />
						{item.text?.length > 0 ?
							<span className="figcaption">{item.text}</span>
						: null}
					</span>
				)
			case "hr":
				return <hr />
			case "blockquote":
				return <blockquote><Children items={item.tokens} /></blockquote>
			case "paragraph":
				return <p><Children items={item.tokens} /></p>
			case "code":
				return <SyntaxHighlighter style={monokai} language={item.lang}>{item.text}</SyntaxHighlighter>
			case "codespan":
				return <code>{item.text}</code>
			case "strong":
				return <b><Children items={item.tokens} /></b>
			case "em":
				return <i><Children items={item.tokens} /></i>
			case "html":
				return <span dangerouslySetInnerHTML={{__html : item.raw}}></span>
			case "br":
				return <br />;
			case "link":
			case "text":
				return <>{item.tokens ? <Children items={item.tokens} /> : processEscapeSequences(item.text)}</>
			case "escape":
				switch(item.text) {
					case "&lt;":
						return '<';
					case "&gt;":
						return '>';
					case '"':
					case "&quot;":
						return '"';
					case "&#39;":
						return "'";
					default:
						return "";
				}
			default:
				return "UNKNOWN ITEM" + JSON.stringify(item);
		}
	}, [item, basePath]);

	return (
		<>
			{content}
		</>
	);
};

export default SlideItem;